@@ -73,7 +73,10 @@ static const gchar *_quark_strings[] = {
   "uri-redirection-permanent", "GstMessagePropertyNotify", "property-name",
   "property-value", "streams", "GstEventSelectStreams",
   "GstMessageStreamCollection", "collection", "stream", "stream-collection",
-  "GstMessageStreamsSelected"
+  "GstMessageStreamsSelected", "GstMessageRedirect", "redirect-entry-locations",
+  "redirect-entry-taglists", "redirect-entry-structures",
+  "GstEventStreamGroupDone", "GstQueryBitrate", "nominal-bitrate",
+  "GstMessageDeviceChanged", "device-changed", "trickmode-interval",
 };
 
 GQuark _priv_gst_quark_table[GST_QUARK_MAX];
