@@ -212,7 +212,17 @@ typedef enum _GstQuarkId
   GST_QUARK_STREAM = 181,
   GST_QUARK_EVENT_STREAM_COLLECTION = 182,
   GST_QUARK_MESSAGE_STREAMS_SELECTED = 183,
-  GST_QUARK_MAX = 184
+  GST_QUARK_MESSAGE_REDIRECT = 184,
+  GST_QUARK_REDIRECT_ENTRY_LOCATIONS = 185,
+  GST_QUARK_REDIRECT_ENTRY_TAGLISTS = 186,
+  GST_QUARK_REDIRECT_ENTRY_STRUCTURES = 187,
+  GST_QUARK_EVENT_STREAM_GROUP_DONE = 188,
+  GST_QUARK_QUERY_BITRATE = 189,
+  GST_QUARK_NOMINAL_BITRATE = 190,
+  GST_QUARK_MESSAGE_DEVICE_CHANGED = 191,
+  GST_QUARK_DEVICE_CHANGED = 192,
+  GST_QUARK_TRICKMODE_INTERVAL = 193,
+  GST_QUARK_MAX = 194
 } GstQuarkId;
 
 extern GQuark _priv_gst_quark_table[GST_QUARK_MAX];
