@@ -89,9 +89,6 @@ GST_DEBUG_CATEGORY_STATIC (task_debug);
 #define SET_TASK_STATE(t,s) (g_atomic_int_set (&GST_TASK_STATE(t), (s)))
 #define GET_TASK_STATE(t)   ((GstTaskState) g_atomic_int_get (&GST_TASK_STATE(t)))
 
-#define GST_TASK_GET_PRIVATE(obj)  \
-   (G_TYPE_INSTANCE_GET_PRIVATE ((obj), GST_TYPE_TASK, GstTaskPrivate))
-
 struct _GstTaskPrivate
 {
   /* callbacks for managing the thread of this task */
@@ -148,26 +145,28 @@ static void gst_task_func (GstTask * task);
 
 static GMutex pool_lock;
 
+static GstTaskPool *_global_task_pool = NULL;
+
 #define _do_init \
 { \
   GST_DEBUG_CATEGORY_INIT (task_debug, "task", 0, "Processing tasks"); \
 }
 
-G_DEFINE_TYPE_WITH_CODE (GstTask, gst_task, GST_TYPE_OBJECT, _do_init);
+G_DEFINE_TYPE_WITH_CODE (GstTask, gst_task, GST_TYPE_OBJECT,
+    G_ADD_PRIVATE (GstTask) _do_init);
 
+/* Called with pool_lock */
 static void
-init_klass_pool (GstTaskClass * klass)
+ensure_klass_pool (GstTaskClass * klass)
 {
-  g_mutex_lock (&pool_lock);
-  if (klass->pool) {
-    gst_task_pool_cleanup (klass->pool);
-    gst_object_unref (klass->pool);
+  if (G_UNLIKELY (_global_task_pool == NULL)) {
+    _global_task_pool = gst_task_pool_new ();
+    gst_task_pool_prepare (_global_task_pool, NULL);
+
+    /* Classes are never destroyed so this ref will never be dropped */
+    GST_OBJECT_FLAG_SET (_global_task_pool, GST_OBJECT_FLAG_MAY_BE_LEAKED);
   }
-  klass->pool = gst_task_pool_new ();
-  /* Classes are never destroyed so this ref will never be dropped */
-  GST_OBJECT_FLAG_SET (klass->pool, GST_OBJECT_FLAG_MAY_BE_LEAKED);
-  gst_task_pool_prepare (klass->pool, NULL);
-  g_mutex_unlock (&pool_lock);
+  klass->pool = _global_task_pool;
 }
 
 static void
@@ -177,11 +176,7 @@ gst_task_class_init (GstTaskClass * klass)
 
   gobject_class = (GObjectClass *) klass;
 
-  g_type_class_add_private (klass, sizeof (GstTaskPrivate));
-
   gobject_class->finalize = gst_task_finalize;
-
-  init_klass_pool (klass);
 }
 
 static void
@@ -191,7 +186,7 @@ gst_task_init (GstTask * task)
 
   klass = GST_TASK_GET_CLASS (task);
 
-  task->priv = GST_TASK_GET_PRIVATE (task);
+  task->priv = gst_task_get_instance_private (task);
   task->running = FALSE;
   task->thread = NULL;
   task->lock = NULL;
@@ -201,11 +196,9 @@ gst_task_init (GstTask * task)
   /* use the default klass pool for this task, users can
    * override this later */
   g_mutex_lock (&pool_lock);
+  ensure_klass_pool (klass);
   task->priv->pool = gst_object_ref (klass->pool);
   g_mutex_unlock (&pool_lock);
-
-  /* clear floating flag */
-  gst_object_ref_sink (task);
 }
 
 static void
@@ -385,7 +378,14 @@ gst_task_cleanup_all (void)
   GstTaskClass *klass;
 
   if ((klass = g_type_class_peek (GST_TYPE_TASK))) {
-    init_klass_pool (klass);
+    if (klass->pool) {
+      g_mutex_lock (&pool_lock);
+      gst_task_pool_cleanup (klass->pool);
+      gst_object_unref (klass->pool);
+      klass->pool = NULL;
+      _global_task_pool = NULL;
+      g_mutex_unlock (&pool_lock);
+    }
   }
 
   /* GstElement owns a GThreadPool */
@@ -423,13 +423,16 @@ gst_task_new (GstTaskFunction func, gpointer user_data, GDestroyNotify notify)
 
   g_return_val_if_fail (func != NULL, NULL);
 
-  task = g_object_newv (GST_TYPE_TASK, 0, NULL);
+  task = g_object_new (GST_TYPE_TASK, NULL);
   task->func = func;
   task->user_data = user_data;
   task->notify = notify;
 
   GST_DEBUG ("Created task %p", task);
 
+  /* clear floating flag */
+  gst_object_ref_sink (task);
+
   return task;
 }
 
