@@ -22,7 +22,7 @@
  * SECTION:gstharness
  * @title: GstHarness
  * @short_description: A test-harness for writing GStreamer unit tests
- * @see_also: #GstTestClock,\
+ * @see_also: #GstTestClock
  *
  * #GstHarness is meant to make writing unit test for GStreamer much easier.
  * It can be thought of as a way of treating a #GstElement as a black box,
@@ -147,6 +147,19 @@ static GstStaticPadTemplate hsinktemplate = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS_ANY);
 
+typedef struct
+{
+  GType api;
+  GstStructure *params;
+} ProposeMeta;
+
+static void
+propose_meta_clear (ProposeMeta * meta)
+{
+  if (meta->params)
+    gst_structure_free (meta->params);
+}
+
 struct _GstHarnessPrivate
 {
   gchar *element_sinkpad_name;
@@ -179,6 +192,8 @@ struct _GstHarnessPrivate
   GstAllocator *propose_allocator;
   GstAllocationParams propose_allocation_params;
 
+  GArray *propose_allocation_metas;
+
   gboolean blocking_push_mode;
   GCond blocking_push_cond;
   GMutex blocking_push_mutex;
@@ -390,6 +405,15 @@ gst_harness_sink_query (GstPad * pad, GstObject * parent, GstQuery * query)
         gst_query_add_allocation_param (query,
             priv->propose_allocator, &priv->propose_allocation_params);
 
+        if (priv->propose_allocation_metas) {
+          guint i;
+          for (i = 0; i < priv->propose_allocation_metas->len; i++) {
+            ProposeMeta *meta =
+                &g_array_index (priv->propose_allocation_metas, ProposeMeta, i);
+            gst_query_add_allocation_meta (query, meta->api, meta->params);
+          }
+        }
+
         GST_DEBUG_OBJECT (pad, "proposing allocation %" GST_PTR_FORMAT,
             priv->propose_allocator);
       }
@@ -912,16 +936,19 @@ gst_harness_add_parse (GstHarness * h, const gchar * launchline)
   GstPad *pad;
   GstIterator *iter;
   gboolean done = FALSE;
+  GError *error = NULL;
 
   g_return_if_fail (launchline != NULL);
 
   desc = g_strdup_printf ("bin.( %s )", launchline);
   bin =
-      (GstBin *) gst_parse_launch_full (desc, NULL, GST_PARSE_FLAG_NONE, NULL);
-  g_free (desc);
+      (GstBin *) gst_parse_launch_full (desc, NULL, GST_PARSE_FLAG_FATAL_ERRORS,
+      &error);
 
-  if (G_UNLIKELY (bin == NULL))
-    return;
+  if (G_UNLIKELY (error != NULL)) {
+    g_error ("Unable to create pipeline '%s': %s", desc, error->message);
+  }
+  g_free (desc);
 
   /* find pads and ghost them if necessary */
   if ((pad = gst_bin_find_unlinked_pad (bin, GST_PAD_SRC)) != NULL) {
@@ -1049,6 +1076,9 @@ gst_harness_teardown (GstHarness * h)
   gst_object_replace ((GstObject **) & priv->allocator, NULL);
   gst_object_replace ((GstObject **) & priv->pool, NULL);
 
+  if (priv->propose_allocation_metas)
+    g_array_unref (priv->propose_allocation_metas);
+
   /* if we hold the last ref, set to NULL */
   if (gst_harness_element_unref (h) == 0) {
     gboolean state_change;
@@ -1079,7 +1109,7 @@ gst_harness_teardown (GstHarness * h)
  * @h: a #GstHarness
  * @srcpad: a #GstPad to link to the harness sinkpad
  *
- * Links the specifed #GstPad the @GstHarness sinkpad. This can be useful if
+ * Links the specified #GstPad the @GstHarness sinkpad. This can be useful if
  * perhaps the srcpad did not exist at the time of creating the harness,
  * like a demuxer that provides a sometimes-pad after receiving data.
  *
@@ -1105,7 +1135,7 @@ gst_harness_add_element_src_pad (GstHarness * h, GstPad * srcpad)
  * @h: a #GstHarness
  * @sinkpad: a #GstPad to link to the harness srcpad
  *
- * Links the specifed #GstPad the @GstHarness srcpad.
+ * Links the specified #GstPad the @GstHarness srcpad.
  *
  * MT safe.
  *
@@ -1149,6 +1179,7 @@ gst_harness_set_src_caps (GstHarness * h, GstCaps * caps)
 
   gst_segment_init (&segment, GST_FORMAT_TIME);
   handled = gst_pad_push_event (h->srcpad, gst_event_new_segment (&segment));
+  g_assert (handled);
 }
 
 /**
@@ -1334,30 +1365,15 @@ gst_harness_set_time (GstHarness * h, GstClockTime time)
  * MT safe.
  *
  * Returns: a @gboolean %TRUE if the waits have been registered, %FALSE if not.
- * (Could be that it timed out waiting or that more waits then waits was found)
+ * (Could be that it timed out waiting or that more waits than waits was found)
  *
  * Since: 1.6
  */
 gboolean
 gst_harness_wait_for_clock_id_waits (GstHarness * h, guint waits, guint timeout)
 {
-  GstTestClock *testclock = h->priv->testclock;
-  gint64 start_time;
-  gboolean ret;
-
-  start_time = g_get_monotonic_time ();
-  while (gst_test_clock_peek_id_count (testclock) < waits) {
-    gint64 time_spent;
-
-    g_usleep (G_USEC_PER_SEC / 1000);
-    time_spent = g_get_monotonic_time () - start_time;
-    if ((time_spent / G_USEC_PER_SEC) > timeout)
-      break;
-  }
-
-  ret = (waits == gst_test_clock_peek_id_count (testclock));
-
-  return ret;
+  return gst_test_clock_timed_wait_for_multiple_pending_ids (h->priv->testclock,
+      waits, timeout * 1000, NULL);
 }
 
 /**
@@ -1370,7 +1386,7 @@ gst_harness_wait_for_clock_id_waits (GstHarness * h, guint waits, guint timeout)
  * 3: Release the #GstClockID wait.
  * Together, this provides an easy way to not have to think about the details
  * around clocks and time, but still being able to write deterministic tests
- * that are dependant on this. A "crank" can be though of as the notion of
+ * that are dependent on this. A "crank" can be though of as the notion of
  * manually driving the clock forward to its next logical step.
  *
  * MT safe.
@@ -1424,7 +1440,7 @@ gst_harness_crank_multiple_clock_waits (GstHarness * h, guint waits)
  *
  * This will set the harnessed #GstElement to %GST_STATE_PLAYING.
  * #GstElements without a sink-#GstPad and with the %GST_ELEMENT_FLAG_SOURCE
- * flag set is concidered a src #GstElement
+ * flag set is considered a src #GstElement
  * Non-src #GstElements (like sinks and filters) are automatically set to
  * playing by the #GstHarness, but src #GstElements are not to avoid them
  * starting to produce buffers.
@@ -1539,7 +1555,8 @@ gst_harness_create_buffer (GstHarness * h, gsize size)
     g_assert_cmpint (flow, ==, GST_FLOW_OK);
     if (gst_buffer_get_size (ret) != size) {
       GST_DEBUG_OBJECT (h,
-          "use fallback, pool is configured with a different size (%zu != %zu)",
+          "use fallback, pool is configured with a different size (%"
+          G_GSIZE_FORMAT " != %" G_GSIZE_FORMAT ")",
           size, gst_buffer_get_size (ret));
       gst_buffer_unref (ret);
       ret = NULL;
@@ -1558,7 +1575,7 @@ gst_harness_create_buffer (GstHarness * h, gsize size)
 /**
  * gst_harness_push:
  * @h: a #GstHarness
- * @buffer: a #GstBuffer to push
+ * @buffer: (transfer full): a #GstBuffer to push
  *
  * Pushes a #GstBuffer on the #GstHarness srcpad. The standard way of
  * interacting with an harnessed element.
@@ -1588,7 +1605,7 @@ gst_harness_push (GstHarness * h, GstBuffer * buffer)
  *
  * MT safe.
  *
- * Returns: a #GstBuffer or %NULL if timed out.
+ * Returns: (transfer full): a #GstBuffer or %NULL if timed out.
  *
  * Since: 1.6
  */
@@ -1618,7 +1635,7 @@ gst_harness_pull (GstHarness * h)
  *
  * MT safe.
  *
- * Returns: a #GstBuffer or %NULL if no buffers are present in the #GAsyncQueue
+ * Returns: (transfer full): a #GstBuffer or %NULL if no buffers are present in the #GAsyncQueue
  *
  * Since: 1.6
  */
@@ -1640,7 +1657,7 @@ gst_harness_try_pull (GstHarness * h)
 /**
  * gst_harness_push_and_pull:
  * @h: a #GstHarness
- * @buffer: a #GstBuffer to push
+ * @buffer: (transfer full): a #GstBuffer to push
  *
  * Basically a gst_harness_push and a gst_harness_pull in one line. Reflects
  * the fact that you often want to do exactly this in your test: Push one buffer
@@ -1648,7 +1665,7 @@ gst_harness_try_pull (GstHarness * h)
  *
  * MT safe.
  *
- * Returns: a #GstBuffer or %NULL if timed out.
+ * Returns: (transfer full): a #GstBuffer or %NULL if timed out.
  *
  * Since: 1.6
  */
@@ -1718,6 +1735,99 @@ gst_harness_set_drop_buffers (GstHarness * h, gboolean drop_buffers)
   priv->drop_buffers = drop_buffers;
 }
 
+/**
+ * gst_harness_take_all_data_as_buffer:
+ * @h: a #GstHarness
+ *
+ * Pulls all pending data from the harness and returns it as a single buffer.
+ *
+ * Returns: (transfer full): the data as a buffer. Unref with gst_buffer_unref()
+ *     when no longer needed.
+ *
+ * Since: 1.14
+ */
+GstBuffer *
+gst_harness_take_all_data_as_buffer (GstHarness * h)
+{
+  GstHarnessPrivate *priv;
+  GstBuffer *ret, *buf;
+
+  g_return_val_if_fail (h != NULL, NULL);
+
+  priv = h->priv;
+
+  g_async_queue_lock (priv->buffer_queue);
+
+  ret = g_async_queue_try_pop_unlocked (priv->buffer_queue);
+
+  if (ret == NULL) {
+    ret = gst_buffer_new ();
+  } else {
+    /* buffer appending isn't very efficient for larger numbers of buffers
+     * or lots of memories, but this function is not performance critical and
+     * we can still improve it if and when the need arises. For now KISS. */
+    while ((buf = g_async_queue_try_pop_unlocked (priv->buffer_queue)))
+      ret = gst_buffer_append (ret, buf);
+  }
+
+  g_async_queue_unlock (priv->buffer_queue);
+
+  return ret;
+}
+
+/**
+ * gst_harness_take_all_data: (skip)
+ * @h: a #GstHarness
+ * @size: (out): the size of the data in bytes
+ *
+ * Pulls all pending data from the harness and returns it as a single
+ * data slice.
+ *
+ * Returns: (transfer full): a pointer to the data, newly allocated. Free
+ *     with g_free() when no longer needed. Will return %NULL if there is no
+ *     data.
+ *
+ * Since: 1.14
+ */
+guint8 *
+gst_harness_take_all_data (GstHarness * h, gsize * size)
+{
+  GstBuffer *buf;
+  guint8 *data = NULL;
+
+  g_return_val_if_fail (h != NULL, NULL);
+  g_return_val_if_fail (size != NULL, NULL);
+
+  buf = gst_harness_take_all_data_as_buffer (h);
+  gst_buffer_extract_dup (buf, 0, -1, (gpointer *) & data, size);
+  gst_buffer_unref (buf);
+  return data;
+}
+
+/**
+ * gst_harness_take_all_data_as_bytes: (rename-to gst_harness_take_all_data)
+ * @h: a #GstHarness
+ *
+ * Pulls all pending data from the harness and returns it as a single #GBytes.
+ *
+ * Returns: (transfer full): a pointer to the data, newly allocated. Free
+ *     with g_free() when no longer needed.
+ *
+ * Since: 1.14
+ */
+GBytes *
+gst_harness_take_all_data_as_bytes (GstHarness * h)
+{
+  guint8 *data;
+  gsize size = 0;
+
+  g_return_val_if_fail (h != NULL, NULL);
+
+  data = gst_harness_take_all_data (h, &size);
+  return g_bytes_new_take (data, size);
+}
+
+
 /**
  * gst_harness_dump_to_file:
  * @h: a #GstHarness
@@ -1733,25 +1843,16 @@ gst_harness_set_drop_buffers (GstHarness * h, gboolean drop_buffers)
 void
 gst_harness_dump_to_file (GstHarness * h, const gchar * filename)
 {
-  GstHarnessPrivate *priv = h->priv;
-  FILE *fd;
-  GstBuffer *buf;
-  fd = fopen (filename, "wb");
-  g_assert (fd);
+  GError *err = NULL;
+  gpointer data;
+  gsize size;
 
-  while ((buf = g_async_queue_try_pop (priv->buffer_queue))) {
-    GstMapInfo info;
-    if (gst_buffer_map (buf, &info, GST_MAP_READ)) {
-      fwrite (info.data, 1, info.size, fd);
-      gst_buffer_unmap (buf, &info);
-    } else {
-      GST_ERROR ("failed to map buffer %p", buf);
-    }
-    gst_buffer_unref (buf);
+  data = gst_harness_take_all_data (h, &size);
+  if (!g_file_set_contents (filename, data ? data : "", size, &err)) {
+    g_error ("GstHarness: Failed to write data to file: %s", err->message);
+    g_clear_error (&err);
   }
-
-  fflush (fd);
-  fclose (fd);
+  g_free (data);
 }
 
 /**
@@ -2074,6 +2175,38 @@ gst_harness_set_propose_allocator (GstHarness * h, GstAllocator * allocator,
     priv->propose_allocation_params = *params;
 }
 
+/**
+ * gst_harness_add_propose_allocation_meta:
+ * @h: a #GstHarness
+ * @api: a metadata API
+ * @params: (allow-none) (transfer none): API specific parameters
+ *
+ * Add api with params as one of the supported metadata API to propose when
+ * receiving an allocation query.
+ *
+ * MT safe.
+ *
+ * Since: 1.16
+ */
+void
+gst_harness_add_propose_allocation_meta (GstHarness * h, GType api,
+    const GstStructure * params)
+{
+  GstHarnessPrivate *priv = h->priv;
+  ProposeMeta meta;
+
+  meta.api = api;
+  meta.params = params ? gst_structure_copy (params) : NULL;
+
+  if (!priv->propose_allocation_metas) {
+    priv->propose_allocation_metas =
+        g_array_new (FALSE, FALSE, sizeof (ProposeMeta));
+    g_array_set_clear_func (priv->propose_allocation_metas,
+        (GDestroyNotify) propose_meta_clear);
+  }
+  g_array_append_val (priv->propose_allocation_metas, meta);
+}
+
 /**
  * gst_harness_add_src_harness:
  * @h: a #GstHarness
@@ -2085,7 +2218,7 @@ gst_harness_set_propose_allocator (GstHarness * h, GstAllocator * allocator,
  * By adding a src-type #GstElement, it is then easy to use functions like
  * gst_harness_push_from_src or gst_harness_src_crank_and_push_many
  * to provide your harnessed element with input. The @has_clock_wait variable
- * is a greate way to control you src-element with, in that you can have it
+ * is a great way to control you src-element with, in that you can have it
  * produce a buffer for you by simply cranking the clock, and not have it
  * spin out of control producing buffers as fast as possible.
  *
@@ -2277,7 +2410,7 @@ forward_sticky_events (GstPad * pad, GstEvent ** ev, gpointer user_data)
  * responses the element output might create in sink elements. An example might
  * be an existing sink providing some analytical data on the input it receives that
  * can be useful to your testing. If the goal is to test a sink-element itself,
- * this is better acheived using gst_harness_new directly on the sink.
+ * this is better achieved using gst_harness_new directly on the sink.
  *
  * If a sink-harness already exists it will be replaced.
  *
@@ -2413,6 +2546,18 @@ gst_harness_find_element (GstHarness * h, const gchar * element_name)
   GstIterator *iter;
   GValue data = G_VALUE_INIT;
 
+  if (!GST_IS_BIN (h->element)) {
+    GstPluginFeature *feature;
+
+    g_return_val_if_fail (GST_IS_ELEMENT (h->element), NULL);
+
+    feature = GST_PLUGIN_FEATURE (gst_element_get_factory (h->element));
+    if (!strcmp (element_name, gst_plugin_feature_get_name (feature)))
+      return gst_object_ref (h->element);
+
+    return NULL;
+  }
+
   iter = gst_bin_iterate_elements (GST_BIN (h->element));
   done = FALSE;
 
