@@ -27,8 +27,7 @@
 
 #include <glib-object.h>
 #include <gst/gst.h>
-
-#include <gst/gstcontrolsource.h>
+#include <gst/controller/controller-prelude.h>
 
 G_BEGIN_DECLS
 
@@ -81,7 +80,7 @@ struct _GstControlPoint
   } cache;
 };
 
-GST_EXPORT
+GST_CONTROLLER_API
 GType gst_control_point_get_type (void);
 
 /**
@@ -116,42 +115,42 @@ struct _GstTimedValueControlSourceClass {
 #define GST_TIMED_VALUE_CONTROL_SOURCE_UNLOCK(o) \
   g_mutex_unlock(&((GstTimedValueControlSource *)o)->lock)
 
-GST_EXPORT
+GST_CONTROLLER_API
 GType           gst_timed_value_control_source_get_type (void);
 
 /* Functions */
 
-GST_EXPORT
+GST_CONTROLLER_API
 GSequenceIter * gst_timed_value_control_source_find_control_point_iter (
                                                                GstTimedValueControlSource * self,
                                                                GstClockTime timestamp);
-GST_EXPORT
+GST_CONTROLLER_API
 gboolean        gst_timed_value_control_source_set            (GstTimedValueControlSource * self,
                                                                GstClockTime timestamp,
                                                                const gdouble value);
-GST_EXPORT
+GST_CONTROLLER_API
 gboolean        gst_timed_value_control_source_set_from_list  (GstTimedValueControlSource * self,
                                                                const GSList * timedvalues);
-GST_EXPORT
+GST_CONTROLLER_API
 gboolean        gst_timed_value_control_source_unset          (GstTimedValueControlSource * self,
                                                                GstClockTime timestamp);
 
-GST_EXPORT
+GST_CONTROLLER_API
 void            gst_timed_value_control_source_unset_all      (GstTimedValueControlSource *self);
 
-GST_EXPORT
+GST_CONTROLLER_API
 GList *         gst_timed_value_control_source_get_all        (GstTimedValueControlSource * self);
 
-GST_EXPORT
+GST_CONTROLLER_API
 gint            gst_timed_value_control_source_get_count      (GstTimedValueControlSource * self);
 
-GST_EXPORT
+GST_CONTROLLER_API
 void            gst_timed_value_control_invalidate_cache      (GstTimedValueControlSource * self);
 
-GST_EXPORT
+GST_CONTROLLER_API
 void            gst_control_point_free (GstControlPoint * cp);
 
-GST_EXPORT
+GST_CONTROLLER_API
 GstControlPoint * gst_control_point_copy (GstControlPoint * cp);
 
 #ifdef G_DEFINE_AUTOPTR_CLEANUP_FUNC
