@@ -21,6 +21,7 @@
  */
 /**
  * SECTION:element-fakesrc
+ * @title: fakesrc
  * @see_also: #GstFakeSink
  *
  * The fakesrc element is a multipurpose element that can generate
@@ -29,13 +30,12 @@
  * It is mostly used as a testing element, one trivial example for testing
  * basic <application>GStreamer</application> core functionality is:
  *
- * <refsect2>
- * <title>Example launch line</title>
+ * ## Example launch line
  * |[
  * gst-launch-1.0 -v fakesrc num-buffers=5 ! fakesink
  * ]| This pipeline will push 5 empty buffers to the fakesink element and then
  * sends an EOS.
- * </refsect2>
+ *
  */
 
 /* FIXME: this ignores basesrc::blocksize property, which could be used as an
@@ -340,8 +340,7 @@ gst_fake_src_class_init (GstFakeSrcClass * klass)
       "Source",
       "Push empty (no data) buffers around",
       "Erik Walthinsen <omega@cse.ogi.edu>, " "Wim Taymans <wim@fluendo.com>");
-  gst_element_class_add_pad_template (gstelement_class,
-      gst_static_pad_template_get (&srctemplate));
+  gst_element_class_add_static_pad_template (gstelement_class, &srctemplate);
 
   gstbase_src_class->is_seekable = GST_DEBUG_FUNCPTR (gst_fake_src_is_seekable);
   gstbase_src_class->start = GST_DEBUG_FUNCPTR (gst_fake_src_start);
@@ -716,7 +715,10 @@ gst_fake_src_create_buffer (GstFakeSrc * src, gsize * bufsize)
         /* try again (this will allocate a new parent) */
         return gst_fake_src_create_buffer (src, bufsize);
       }
-      gst_buffer_map (buf, &info, GST_MAP_WRITE);
+      if (buf == NULL)
+        goto buffer_create_fail;
+      if (!gst_buffer_map (buf, &info, GST_MAP_WRITE))
+        goto buffer_write_fail;
       gst_fake_src_prepare_buffer (src, info.data, info.size);
       gst_buffer_unmap (buf, &info);
       break;
@@ -726,12 +728,28 @@ gst_fake_src_create_buffer (GstFakeSrc * src, gsize * bufsize)
       break;
   }
   if (dump) {
-    gst_buffer_map (buf, &info, GST_MAP_READ);
-    gst_util_dump_mem (info.data, info.size);
-    gst_buffer_unmap (buf, &info);
+    if (gst_buffer_map (buf, &info, GST_MAP_READ)) {
+      gst_util_dump_mem (info.data, info.size);
+      gst_buffer_unmap (buf, &info);
+    }
   }
 
   return buf;
+
+buffer_create_fail:
+  {
+    GST_ELEMENT_ERROR (src, RESOURCE, BUSY, (NULL),
+        ("Failed to create a buffer"));
+    return NULL;
+  }
+
+buffer_write_fail:
+  {
+    GST_ELEMENT_ERROR (src, RESOURCE, WRITE, (NULL),
+        ("Failed to write to buffer"));
+    gst_buffer_unref (buf);
+    return NULL;
+  }
 }
 
 static void
