@@ -66,12 +66,11 @@ struct _GstTee {
   GstElement      element;
 
   /*< private >*/
-  /* lock protecting dynamic pads */
-  GMutex          dyn_lock;
-
   GstPad         *sinkpad;
   GstPad         *allocpad;
-  guint           pad_counter;
+
+  GHashTable     *pad_indexes;
+  guint           next_pad_index;
 
   gboolean        has_chain;
   gboolean        silent;
@@ -80,6 +79,8 @@ struct _GstTee {
   GstPadMode      sink_mode;
   GstTeePullMode  pull_mode;
   GstPad         *pull_pad;
+
+  gboolean        allow_not_linked;
 };
 
 struct _GstTeeClass {
