@@ -68,7 +68,7 @@ static gboolean waiting_eos = FALSE;
 static gchar **exclude_args = NULL;
 
 /* convenience macro so we don't have to litter the code with if(!quiet) */
-#define PRINT if(!quiet)g_print
+#define PRINT if(!quiet)gst_print
 
 #ifdef G_OS_UNIX
 static void
@@ -76,7 +76,7 @@ fault_handler_sighandler (int signum)
 {
   fault_restore ();
 
-  /* printf is used instead of g_print(), since it's less likely to
+  /* printf is used instead of gst_print(), since it's less likely to
    * deadlock */
   switch (signum) {
     case SIGSEGV:
@@ -249,13 +249,13 @@ print_index_stats (GPtrArray * index_stats)
   gint i;
 
   if (index_stats->len) {
-    g_print ("%s:\n", _("Index statistics"));
+    gst_print ("%s:\n", _("Index statistics"));
   }
 
   for (i = 0; i < index_stats->len; i++) {
     GstIndexStats *s = g_ptr_array_index (index_stats, i);
     if (s) {
-      g_print ("id %d, %s\n", s->id, s->desc);
+      gst_print ("id %d, %s\n", s->id, s->desc);
       if (s->num_frames) {
         GstClockTime last_frame = s->last_keyframe;
 
@@ -266,24 +266,24 @@ print_index_stats (GPtrArray * index_stats)
         }
 
         if (GST_CLOCK_TIME_IS_VALID (last_frame)) {
-          g_print ("  total time               = %" GST_TIME_FORMAT "\n",
+          gst_print ("  total time               = %" GST_TIME_FORMAT "\n",
               GST_TIME_ARGS (last_frame));
         }
-        g_print ("  frame/keyframe rate      = %u / %u = ", s->num_frames,
+        gst_print ("  frame/keyframe rate      = %u / %u = ", s->num_frames,
             s->num_keyframes);
         if (s->num_keyframes)
-          g_print ("%lf\n", s->num_frames / (gdouble) s->num_keyframes);
+          gst_print ("%lf\n", s->num_frames / (gdouble) s->num_keyframes);
         else
-          g_print ("-\n");
+          gst_print ("-\n");
         if (s->num_keyframes) {
-          g_print ("  min/avg/max keyframe gap = %" GST_TIME_FORMAT ", %"
+          gst_print ("  min/avg/max keyframe gap = %" GST_TIME_FORMAT ", %"
               GST_TIME_FORMAT ", %" GST_TIME_FORMAT "\n",
               GST_TIME_ARGS (s->min_keyframe_gap),
               GST_TIME_ARGS (s->avg_keyframe_gap),
               GST_TIME_ARGS (s->max_keyframe_gap));
         }
       } else {
-        g_print ("  no stats\n");
+        gst_print ("  no stats\n");
       }
 
       g_free (s->desc);
@@ -322,9 +322,9 @@ print_error_message (GstMessage * msg)
   name = gst_object_get_path_string (msg->src);
   gst_message_parse_error (msg, &err, &debug);
 
-  g_printerr (_("ERROR: from element %s: %s\n"), name, err->message);
+  gst_printerr (_("ERROR: from element %s: %s\n"), name, err->message);
   if (debug != NULL)
-    g_printerr (_("Additional debug info:\n%s\n"), debug);
+    gst_printerr (_("Additional debug info:\n%s\n"), debug);
 
   g_clear_error (&err);
   g_free (debug);
@@ -425,7 +425,7 @@ print_tag_foreach (const GstTagList * tags, const gchar * tag,
   else
     str = gst_value_serialize (&val);
 
-  g_print ("%*s%s: %s\n", 2 * depth, " ", gst_tag_get_nick (tag), str);
+  gst_print ("%*s%s: %s\n", 2 * depth, " ", gst_tag_get_nick (tag), str);
   g_free (str);
 
   g_value_unset (&val);
@@ -899,7 +899,7 @@ event_loop (GstElement * pipeline, gboolean blocking, gboolean do_progress,
           val_str = g_strdup ("(no value)");
         }
 
-        g_print ("%s: %s = %s\n", obj_name, name, val_str);
+        gst_print ("%s: %s = %s\n", obj_name, name, val_str);
         g_free (obj_name);
         g_free (val_str);
         break;
@@ -1044,9 +1044,9 @@ main (int argc, char *argv[])
   g_option_context_add_group (ctx, gst_init_get_option_group ());
   if (!g_option_context_parse (ctx, &argc, &argv, &err)) {
     if (err)
-      g_printerr ("Error initializing: %s\n", GST_STR_NULL (err->message));
+      gst_printerr ("Error initializing: %s\n", GST_STR_NULL (err->message));
     else
-      g_printerr ("Error initializing: Unknown error!\n");
+      gst_printerr ("Error initializing: Unknown error!\n");
     g_clear_error (&err);
     g_option_context_free (ctx);
     exit (1);
@@ -1074,22 +1074,21 @@ main (int argc, char *argv[])
 
   if (!pipeline) {
     if (error) {
-      g_printerr (_("ERROR: pipeline could not be constructed: %s.\n"),
+      gst_printerr (_("ERROR: pipeline could not be constructed: %s.\n"),
           GST_STR_NULL (error->message));
       g_clear_error (&error);
     } else {
-      g_printerr (_("ERROR: pipeline could not be constructed.\n"));
+      gst_printerr (_("ERROR: pipeline could not be constructed.\n"));
     }
     return 1;
   } else if (error) {
-    g_printerr (_("WARNING: erroneous pipeline: %s\n"),
+    gst_printerr (_("WARNING: erroneous pipeline: %s\n"),
         GST_STR_NULL (error->message));
     g_clear_error (&error);
     return 1;
   }
 
   if (!savefile) {
-    GstState state, pending;
     GstStateChangeReturn ret;
     GstBus *bus;
 
@@ -1098,7 +1097,7 @@ main (int argc, char *argv[])
       GstElement *real_pipeline = gst_element_factory_make ("pipeline", NULL);
 
       if (real_pipeline == NULL) {
-        g_printerr (_("ERROR: the 'pipeline' element wasn't found.\n"));
+        gst_printerr (_("ERROR: the 'pipeline' element wasn't found.\n"));
         return 1;
       }
       gst_bin_add (GST_BIN (real_pipeline), pipeline);
@@ -1134,7 +1133,7 @@ main (int argc, char *argv[])
 
     switch (ret) {
       case GST_STATE_CHANGE_FAILURE:
-        g_printerr (_("ERROR: Pipeline doesn't want to pause.\n"));
+        gst_printerr (_("ERROR: Pipeline doesn't want to pause.\n"));
         res = -1;
         event_loop (pipeline, FALSE, FALSE, GST_STATE_VOID_PENDING);
         goto end;
@@ -1146,11 +1145,10 @@ main (int argc, char *argv[])
         PRINT (_("Pipeline is PREROLLING ...\n"));
         caught_error = event_loop (pipeline, TRUE, TRUE, GST_STATE_PAUSED);
         if (caught_error) {
-          g_printerr (_("ERROR: pipeline doesn't want to preroll.\n"));
+          gst_printerr (_("ERROR: pipeline doesn't want to preroll.\n"));
           res = caught_error;
           goto end;
         }
-        state = GST_STATE_PAUSED;
         /* fallthrough */
       case GST_STATE_CHANGE_SUCCESS:
         PRINT (_("Pipeline is PREROLLED ...\n"));
@@ -1160,7 +1158,7 @@ main (int argc, char *argv[])
     caught_error = event_loop (pipeline, FALSE, TRUE, GST_STATE_PLAYING);
 
     if (caught_error) {
-      g_printerr (_("ERROR: pipeline doesn't want to preroll.\n"));
+      gst_printerr (_("ERROR: pipeline doesn't want to preroll.\n"));
       res = caught_error;
     } else {
       GstClockTime tfthen, tfnow;
@@ -1173,7 +1171,7 @@ main (int argc, char *argv[])
         GstMessage *err_msg;
         GstBus *bus;
 
-        g_printerr (_("ERROR: pipeline doesn't want to play.\n"));
+        gst_printerr (_("ERROR: pipeline doesn't want to play.\n"));
         bus = gst_element_get_bus (pipeline);
         if ((err_msg = gst_bus_poll (bus, GST_MESSAGE_ERROR, 0))) {
           print_error_message (err_msg);
@@ -1230,22 +1228,10 @@ main (int argc, char *argv[])
           GST_TIME_ARGS (diff));
     }
 
-    PRINT (_("Setting pipeline to PAUSED ...\n"));
-    gst_element_set_state (pipeline, GST_STATE_PAUSED);
-    if (caught_error == ELR_NO_ERROR)
-      gst_element_get_state (pipeline, &state, &pending, GST_CLOCK_TIME_NONE);
-
-    /* iterate mainloop to process pending stuff */
-    while (g_main_context_iteration (NULL, FALSE));
-
     /* No need to see all those pad caps going to NULL etc., it's just noise */
     if (deep_notify_id != 0)
       g_signal_handler_disconnect (pipeline, deep_notify_id);
 
-    PRINT (_("Setting pipeline to READY ...\n"));
-    gst_element_set_state (pipeline, GST_STATE_READY);
-    gst_element_get_state (pipeline, &state, &pending, GST_CLOCK_TIME_NONE);
-
 #if 0
     if (check_index) {
       print_index_stats (index_stats);
